import { Nullable, Arrayable } from '@stacksjs/types';
import { Macroable } from 'macroable';

declare function average(arr: number[]): number;
declare function avg(arr: number[]): number;
declare function median(arr: number[]): number;
declare function mode(arr: number[]): number;
declare function sum(array: readonly number[]): number;

/**
 * Convert `Arrayable<T>` to `Array<T>`
 *
 * @category Array
 */
declare function toArray<T>(array?: Nullable<Arrayable<T>>): Array<T>;
/**
 * Convert `Arrayable<T>` to `Array<T>` and flatten it
 *
 * @category Array
 */
declare function flatten<T>(array?: Nullable<Arrayable<T | Array<T>>>): Array<T>;
/**
 * Use rest arguments to merge arrays
 *
 * @category Array
 */
declare function mergeArrayable<T>(...args: Nullable<Arrayable<T>>[]): Array<T>;
type PartitionFilter<T> = (i: T, idx: number, arr: readonly T[]) => any;
/**
 * Divide an array into two parts by a filter function
 *
 * @category Array
 * @example const [odd, even] = partition([1, 2, 3, 4], i => i % 2 != 0)
 */
declare function partition<T>(array: readonly T[], f1: PartitionFilter<T>): [T[], T[]];
declare function partition<T>(array: readonly T[], f1: PartitionFilter<T>, f2: PartitionFilter<T>): [T[], T[], T[]];
declare function partition<T>(array: readonly T[], f1: PartitionFilter<T>, f2: PartitionFilter<T>, f3: PartitionFilter<T>): [T[], T[], T[], T[]];
declare function partition<T>(array: readonly T[], f1: PartitionFilter<T>, f2: PartitionFilter<T>, f3: PartitionFilter<T>, f4: PartitionFilter<T>): [T[], T[], T[], T[], T[]];
declare function partition<T>(array: readonly T[], f1: PartitionFilter<T>, f2: PartitionFilter<T>, f3: PartitionFilter<T>, f4: PartitionFilter<T>, f5: PartitionFilter<T>): [T[], T[], T[], T[], T[], T[]];
declare function partition<T>(array: readonly T[], f1: PartitionFilter<T>, f2: PartitionFilter<T>, f3: PartitionFilter<T>, f4: PartitionFilter<T>, f5: PartitionFilter<T>, f6: PartitionFilter<T>): [T[], T[], T[], T[], T[], T[], T[]];
/**
 * Unique an Array
 *
 * @category Array
 */
declare function uniq<T>(array: readonly T[]): T[];
declare function unique<T>(array: readonly T[]): T[];
/**
 * Unique an Array by a custom equality function
 *
 * @category Array
 */
declare function uniqueBy<T>(array: readonly T[], equalFn: (a: any, b: any) => boolean): T[];
/**
 * Get last item
 *
 * @category Array
 */
declare function last(array: readonly []): undefined;
declare function last<T>(array: readonly T[]): T;
/**
 * Remove an item from Array
 *
 * @category Array
 */
declare function remove<T>(array: T[], value: T): boolean;
/**
 * Get nth item of Array. Negative for backward
 *
 * @category Array
 */
declare function at(array: readonly [], index: number): undefined;
declare function at<T>(array: readonly T[], index: number): T;
/**
 * Generate a range array of numbers. The `stop` is exclusive.
 *
 * @category Array
 */
declare function range(stop: number): number[];
declare function range(start: number, stop: number, step?: number): number[];
/**
 * Move element in an Array
 *
 * @category Array
 * @param arr
 * @param from
 * @param to
 */
declare function move<T>(arr: T[], from: number, to: number): T[];
/**
 * Clamp a number to the index range of an array.
 *
 * @category Array
 */
declare function clampArrayRange(arr: readonly unknown[], n: number): number;
/**
 * Get random items from an array
 *
 * @category Array
 */
declare function sample<T>(arr: T[], count: number): T[];
/**
 * Shuffle an array. This function mutates the array.
 *
 * @category Array
 */
declare function shuffle<T>(array: T[]): T[];

declare function contains(needle: string, haystack: string[]): boolean;
declare function containsAll(needles: string[], haystack: string[]): boolean;
declare function containsAny(needles: string[], haystack: string[]): boolean;
declare function containsNone(needles: string[], haystack: string[]): boolean;
declare function containsOnly(needles: string[], haystack: string[]): boolean;
declare function doesNotContain(needle: string, haystack: string[]): boolean;

declare class Arr extends Macroable {
    contains(needle: string, haystack: string[]): boolean;
    containsAll(needles: string[], haystack: string[]): boolean;
    containsAny(needles: string[], haystack: string[]): boolean;
    containsNone(needles: string[], haystack: string[]): boolean;
    containsOnly(needles: string[], haystack: string[]): boolean;
    doesNotContain(needle: string, haystack: string[]): boolean;
    toArray<T>(array?: Nullable<Arrayable<T>>): Array<T>;
    flatten<T>(array?: Nullable<Arrayable<T | Array<T>>>): Array<T>;
    mergeArrayable<T>(...args: Nullable<Arrayable<T>>[]): Array<T>;
    partition<T>(array: readonly T[], filter: PartitionFilter<T>): [T[], T[]];
    /**
     * Returns a random item/s from the array
     */
    random<T>(arr: T[], count?: number): T[];
    /**
     * Returns random item/s from the array
     */
    sample<T>(arr: T[], count?: number): T[];
    unique<T>(arr: T[]): T[];
    uniqueBy<T>(arr: readonly T[], equalFn: (a: any, b: any) => boolean): T[];
    last<T>(arr: T[]): T | undefined;
    remove<T>(arr: T[], value: T): boolean;
    at(arr: any[], index: number): any;
    range(start: number, end: number, step?: number): number[];
    move<T>(arr: T[], from: number, to: number): T[];
    clampArrayRange(arr: readonly unknown[], n: number): number;
    shuffle<T>(arr: T[]): T[];
    /**
     * Returns the sum of all items in the array
     */
    sum(arr: number[]): number;
    /**
     * Returns the average of all items in the array
     */
    average(arr: number[]): number;
    avg(arr: number[]): number;
    /**
     * Returns the median of all items in the array
     */
    median(arr: number[]): number;
    /**
     * Returns the mode of all items in the array
     */
    mode(arr: number[]): number;
    static macros: {};
    static getters: {};
}
declare const arr: Arr;

export { Arr, PartitionFilter, arr, at, average, avg, clampArrayRange, contains, containsAll, containsAny, containsNone, containsOnly, doesNotContain, flatten, last, median, mergeArrayable, mode, move, partition, range, remove, sample, shuffle, sum, toArray, uniq, unique, uniqueBy };
