import { clamp } from '@stacksjs/utils';
import { Macroable } from 'macroable';

function average(arr) {
  return sum(arr) / arr.length;
}
function avg(arr) {
  return average(arr);
}
function median(arr) {
  return arr[Math.floor(arr.length / 2)];
}
function mode(arr) {
  return arr.sort((a, b) => arr.filter((v) => v === a).length - arr.filter((v) => v === b).length).pop();
}
function sum(array) {
  return array.reduce((acc, cur) => acc + cur, 0);
}

function toArray(array) {
  array = array ?? [];
  return Array.isArray(array) ? array : [array];
}
function flatten(array) {
  return toArray(array).flat(1);
}
function mergeArrayable(...args) {
  return args.flatMap((i) => toArray(i));
}
function partition(array, ...filters) {
  const result = new Array(filters.length + 1).fill(null).map(() => []);
  array.forEach((e, idx, arr) => {
    let i = 0;
    for (const filter of filters) {
      if (filter(e, idx, arr)) {
        result[i].push(e);
        return;
      }
      i += 1;
    }
    result[i].push(e);
  });
  return result;
}
function uniq(array) {
  return Array.from(new Set(array));
}
function unique(array) {
  return uniq(array);
}
function uniqueBy(array, equalFn) {
  return array.reduce((acc, cur) => {
    const index = acc.findIndex((item) => equalFn(cur, item));
    if (index === -1)
      acc.push(cur);
    return acc;
  }, []);
}
function last(array) {
  return at(array, -1);
}
function remove(array, value) {
  if (!array)
    return false;
  const index = array.indexOf(value);
  if (index >= 0) {
    array.splice(index, 1);
    return true;
  }
  return false;
}
function at(array, index) {
  const len = array.length;
  if (!len)
    return void 0;
  if (index < 0)
    index += len;
  return array[index];
}
function range(...args) {
  let start, stop, step;
  if (args.length === 1) {
    start = 0;
    step = 1;
    [stop] = args;
  } else {
    [start, stop, step = 1] = args;
  }
  const arr = [];
  let current = start;
  while (current < stop) {
    arr.push(current);
    current += step || 1;
  }
  return arr;
}
function move(arr, from, to) {
  arr.splice(to, 0, arr.splice(from, 1)[0]);
  return arr;
}
function clampArrayRange(arr, n) {
  return clamp(n, 0, arr.length - 1);
}
function sample(arr, count) {
  return Array.from({ length: count }, (_) => arr[Math.round(Math.random() * (arr.length - 1))]);
}
function shuffle(array) {
  for (let i = array.length - 1; i > 0; i--) {
    const j = Math.floor(Math.random() * (i + 1));
    [array[i], array[j]] = [array[j], array[i]];
  }
  return array;
}

function contains(needle, haystack) {
  return haystack.some((hay) => needle.includes(hay));
}
function containsAll(needles, haystack) {
  return needles.every((needle) => contains(needle, haystack));
}
function containsAny(needles, haystack) {
  return needles.some((needle) => contains(needle, haystack));
}
function containsNone(needles, haystack) {
  return !containsAny(needles, haystack);
}
function containsOnly(needles, haystack) {
  return containsAll(haystack, needles);
}
function doesNotContain(needle, haystack) {
  return !contains(needle, haystack);
}

class Arr extends Macroable {
  contains(needle, haystack) {
    return contains(needle, haystack);
  }
  containsAll(needles, haystack) {
    return containsAll(needles, haystack);
  }
  containsAny(needles, haystack) {
    return containsAny(needles, haystack);
  }
  containsNone(needles, haystack) {
    return containsNone(needles, haystack);
  }
  containsOnly(needles, haystack) {
    return containsOnly(needles, haystack);
  }
  doesNotContain(needle, haystack) {
    return doesNotContain(needle, haystack);
  }
  toArray(array) {
    return toArray(array);
  }
  flatten(array) {
    return flatten(array);
  }
  mergeArrayable(...args) {
    return mergeArrayable(...args);
  }
  partition(array, filter) {
    return partition(array, filter);
  }
  /**
   * Returns a random item/s from the array
   */
  random(arr2, count = 1) {
    return sample(arr2, count);
  }
  /**
   * Returns random item/s from the array
   */
  sample(arr2, count = 1) {
    return sample(arr2, count);
  }
  unique(arr2) {
    return uniq(arr2);
  }
  uniqueBy(arr2, equalFn) {
    return uniqueBy(arr2, equalFn);
  }
  last(arr2) {
    return last(arr2);
  }
  remove(arr2, value) {
    return remove(arr2, value);
  }
  at(arr2, index) {
    return at(arr2, index);
  }
  range(start, end, step = 1) {
    return range(start, end, step);
  }
  move(arr2, from, to) {
    return move(arr2, from, to);
  }
  clampArrayRange(arr2, n) {
    return clampArrayRange(arr2, n);
  }
  shuffle(arr2) {
    return shuffle(arr2);
  }
  /**
   * Returns the sum of all items in the array
   */
  sum(arr2) {
    return sum(arr2);
  }
  /**
   * Returns the average of all items in the array
   */
  average(arr2) {
    return average(arr2);
  }
  avg(arr2) {
    return average(arr2);
  }
  /**
   * Returns the median of all items in the array
   */
  median(arr2) {
    return median(arr2);
  }
  /**
   * Returns the mode of all items in the array
   */
  mode(arr2) {
    return mode(arr2);
  }
}
Arr.macros = {};
Arr.getters = {};
const arr = new Arr();

export { Arr, arr, at, average, avg, clampArrayRange, contains, containsAll, containsAny, containsNone, containsOnly, doesNotContain, flatten, last, median, mergeArrayable, mode, move, partition, range, remove, sample, shuffle, sum, toArray, uniq, unique, uniqueBy };
